
/**
 * Ein rechteckiges Objekt im Dorianland mit vier unsichtbaren Begrenzungen:
 * - linker Rand durch eine schwarze Linie
 * - rechter Rand durch eine weiße Linie
 * - oberer Rand durch eine hellgrüne Linie
 * - unterer Rand durch eine hellgelbe Linie
 * 
 * @author Johannes Neumeyer
 * @version 1.0
 */
public class DLRechteck extends Rechteck
{
    private Rechteck randLinks;
    private Rechteck randRechts;
    private Rechteck randOben;
    private Rechteck randUnten;
    private Text koordinaten;

    /**
     * Konstruktor für Objekte der Klasse DLRechteck
     */
    public DLRechteck()
    {
        super();

        randLinks = new Rechteck();
        randLinks.PositionSetzen(x, y + 10);
        randLinks.GrößeSetzen(1, höhe - 10);
        randLinks.FarbeSetzen("schwarz");

        randRechts = new Rechteck();
        randRechts.PositionSetzen(x + breite - 1, y + 10);
        randRechts.GrößeSetzen(1, höhe - 10);
        randRechts.FarbeSetzen("weiß");

        randOben = new Rechteck();
        randOben.PositionSetzen(x + 3, y);
        randOben.GrößeSetzen(breite - 6, 10);
        randOben.FarbeSetzen("hellgrün");
        
        randUnten = new Rechteck();
        randUnten.PositionSetzen(x + 3, y + höhe - 10);
        randUnten.GrößeSetzen(breite - 6, 10);
        randUnten.FarbeSetzen("hellgelb");

        randLinks.GanzNachHintenBringen();
        randRechts.GanzNachHintenBringen();
        randOben.GanzNachHintenBringen();
        randUnten.GanzNachHintenBringen();       

        koordinaten = new Text();
        koordinaten.TextGrößeSetzen(10);
        KoordinatentextAktualisieren();
        koordinaten.PositionSetzen(x, y);
        koordinaten.FarbeSetzen("schwarz");
        koordinaten.NachVornBringen();
    }

    /**
     * Überschrieben, da auch die Positionen der unsichtbaren Ränder
     * und der Koordinatentext aktualisiert werden müssen.
     * @param xNeu die (neue) x-Koordinate
     * @param yNeu die (neue) y-Koordinate
     */
    @Override void PositionSetzen(int xNeu, int yNeu)
    {
        super.PositionSetzen(xNeu, yNeu);
        randLinks.PositionSetzen(xNeu, y + 10);
        randRechts.PositionSetzen(xNeu + breite - 1, y + 10);
        randOben.PositionSetzen(xNeu + 3, yNeu);
        randUnten.PositionSetzen(xNeu + 3, yNeu + höhe - 10);
        koordinaten.PositionSetzen(xNeu, yNeu);
        KoordinatentextAktualisieren();
    }

    /**
     * Überschrieben, da auch die Größen/Positionen der unsichtbaren Ränder
     * aktualisiert werden müssen.
     * @param breiteNeu die (neue) Breite
     * @param höheNeu die (neue) Höhe
     */
    @Override void GrößeSetzen(int breiteNeu, int höheNeu)
    {
        super.GrößeSetzen(breiteNeu, höheNeu);
        randLinks.GrößeSetzen(randLinks.breite, höheNeu - 10);
        randRechts.GrößeSetzen(randRechts.breite, höheNeu - 10);
        randRechts.PositionSetzen(x + breite - 1, y + 10);
        randOben.GrößeSetzen(breiteNeu - 6, 10);
        randUnten.GrößeSetzen(breiteNeu - 6, 10);
        randUnten.PositionSetzen(x + 3, y + höhe - 10);
    }

    /**
     * Der Text, der die Koordinaten des Rechtecks angibt, wird aktualisiert.
     */
    private void KoordinatentextAktualisieren()
    {
        koordinaten.TextSetzen("(" + x + "; " + y + ")");
    }
}
