// WARNING: This file is auto-generated and any changes to it will be overwritten
import lang.stride.*;
import java.util.*;

/**
 * Verwaltet die Fahrzeuge auf einer Fahrbahn.
 * @author Albert Wiedemann @version V1.0
 */
public class Fahrbahn extends Ereignisbehandlung
{
    /* x-Position*/
    private int x;
    /* y-Position*/
    private int y;
    /* start-Position*/
    private int start;
    /* ende-Position*/
    private int ende;
    /* rand-Position*/
    private int rand;
    /* Wartezeit zur nächsten Fahrzeugerzeugung*/
    private int wartezeit;
    /* erstes Fahrzeug*/
    private Fahrzeug fahrzeug1;
    /* zweites Fahrzeug*/
    private Fahrzeug fahrzeug2;
    /* drittes Fahrzeug*/
    private Fahrzeug fahrzeug3;
    /* Darstellungssymbol*/
    private FahrbahnSymbol darstellung;
    /* zuständige Ampel*/
    private Ampel ampel;

    /**
     * Erzeugt eine leere Fahrbahn ab dem linken Rand unterhalb der Fenstermitte. @param a für diese Richtung zuständige Ampel @param x x-Koordinate der Fahrbahn @param y y-Koordinate der Fahrbahn
     */
    protected Fahrbahn(Ampel a, int x, int y)
    {
        this.x = x;
        this.y = y;
        ampel = a;
        fahrzeug1 = null;
        fahrzeug2 = null;
        fahrzeug3 = null;
        darstellung =  new  FahrbahnSymbol();
        darstellung.PositionSetzen(x, y);
        start = darstellung.EintrittsPositionGeben();
        ende = darstellung.AustrittsPositionGeben();
        rand = darstellung.RandAbstandGeben();
        wartezeit = 0;
    }

    /**
     * Reiht ein neues Fahrzeug an der Startposition in die Fahrbahn ein. @param f das neue Fahrzeug
     */
    private void FahrzeugEinreihen(Fahrzeug f)
    {
        if (fahrzeug1 == null) {
            fahrzeug1 = f;
        }
        else if (fahrzeug2 == null) {
            fahrzeug2 = f;
        }
        else if (fahrzeug3 == null) {
            fahrzeug3 = f;
        }
        f.PositionSetzen(x + rand, start);
    }

    /**
     * Ermittelt, ob ein Fahrzeug entfernt werden muss @param f das in Frage kommende Fahrzeug
     */
    private boolean IstDraussen(Fahrzeug f)
    {
        return f.YKoordinateGeben() < ende;
    }

    /**
     * Ermittelt, ob das Fahrzeug sich nach dem Vorgänger richten oder die Ampel beachten muss. @param ampelabstand Abstand zur Ampel (Haltelinie) @param vorgängerabstand Abstand zum Vorgängerfahrzeug @param f das aktuelle Fahrzeug @return true, wenn der Vorgänger wichtig ist, false, wenn die Ampel zählt
     */
    private boolean VorgängerZählt(int ampelabstand, int vorgängerabstand, Fahrzeug f)
    {
        return (ampelabstand < 0) || (vorgängerabstand < ampelabstand) || ampel.IstGrün() || ampel.IstGelb() && (ampelabstand < f.GeschwindigkeitGeben() * 1.8 * 5) && (f.GeschwindigkeitGeben() > 3.0);
    }

    /**
     * Bewegt alle Fahrzeuge der Fahrbahn für eine Sekunde.
     */
    private void FahrzeugeBewegen()
    {
        double vvorgänger;
        int wegmax;
        int ampelAbstand;
        /* 1. Fahrzeug*/
        if (fahrzeug1 != null) {
            wegmax = 100000;
            vvorgänger = 100;
            ampelAbstand = fahrzeug1.YKoordinateGeben() - darstellung.HaltelinienPositionGeben();
            if (VorgängerZählt(ampelAbstand, wegmax, fahrzeug1)) {
                fahrzeug1.Fahren(wegmax, vvorgänger);
            }
            else {
                fahrzeug1.Fahren(ampelAbstand, 0.0);
            }
            /* 2. Fahrzeug*/
            if (fahrzeug2 != null) {
                vvorgänger = fahrzeug1.GeschwindigkeitGeben();
                wegmax = fahrzeug2.YKoordinateGeben() - fahrzeug1.YKoordinateGeben() - fahrzeug1.LängeGeben() - 2;
                if (vvorgänger > 0) {
                    wegmax = wegmax - (int)Math.round((fahrzeug2.GeschwindigkeitGeben() + vvorgänger) / 2.0 * 1.8 * 5);
                }
                ampelAbstand = fahrzeug2.YKoordinateGeben() - darstellung.HaltelinienPositionGeben();
                if (VorgängerZählt(ampelAbstand, wegmax, fahrzeug2)) {
                    fahrzeug2.Fahren(wegmax, vvorgänger);
                }
                else {
                    fahrzeug2.Fahren(ampelAbstand, 0.0);
                }
                /* 3. Fahrzeug*/
                if (fahrzeug3 != null) {
                    vvorgänger = fahrzeug2.GeschwindigkeitGeben();
                    wegmax = fahrzeug3.YKoordinateGeben() - fahrzeug2.YKoordinateGeben() - fahrzeug2.LängeGeben() - 2;
                    if (vvorgänger > 0) {
                        wegmax = wegmax - (int)Math.round((fahrzeug3.GeschwindigkeitGeben() + vvorgänger) / 2.0 * 1.8 * 5);
                    }
                    ampelAbstand = fahrzeug3.YKoordinateGeben() - darstellung.HaltelinienPositionGeben();
                    if (VorgängerZählt(ampelAbstand, wegmax, fahrzeug3)) {
                        fahrzeug3.Fahren(wegmax, vvorgänger);
                    }
                    else {
                        fahrzeug3.Fahren(ampelAbstand, 0.0);
                    }
                }
            }
        }
        /* Erstes Fahrzeug draußen?*/
        if ((fahrzeug1 != null) && IstDraussen(fahrzeug1)) {
            fahrzeug1.Entfernen();
            fahrzeug1 = fahrzeug2;
            fahrzeug2 = fahrzeug3;
            fahrzeug3 = null;
        }
    }

    /**
     * Setzt die Position der Fahrbahn. @param x x-Position @param y y-Position
     */
    protected void PositionSetzen(int x, int y)
    {
        this.x = x;
        this.y = y;
        darstellung.PositionSetzen(x, y);
        rand = darstellung.RandAbstandGeben();
    }

    /**
     * Ausgabe der Fahrzeuganzahl @return Anzahl der Fahrzeuge auf der Fahrbahn
     */
    private int FahrzeuganzahlGeben()
    {
        if (fahrzeug1 == null) {
            return 0;
        }
        else if (fahrzeug2 == null) {
            return 1;
        }
        else if (fahrzeug3 == null) {
            return 2;
        }
        else {
            return 3;
        }
    }

    /**
     * Fährt die Fahrzeuge und erzeugt gegebenenfalls ein neues Fahrzeug.
     */
    protected void TaktImpulsAusführen()
    {
        FahrzeugeBewegen();
        if (wartezeit <= 0) {
            FahrzeugEinreihen( new  Fahrzeug());
            wartezeit = 4;
        }
        else {
            wartezeit = wartezeit - 1;
        }
    }
}
