
/**
 * Beschreibt ein Fahrzeug.
 * 
 * @author Albert Wiedemann
 * @version V1.0
 */
class Fahrzeug
{
    /** Anzeigesymbol */
    private FahrzeugSymbol symbol;
    // Bewegungsdaten
    /** x-Position */
    private int x;
    /** y-Position */
    private int y;
    /** Geschwindigkeit */
    private double v;
    /** Maximale Geschwindigkeit */
    private double vmax;
    /** Beschleunigung */
    private double a;

    /**
     * Erzeugt die Anzeige und setzt die Startdaten
     */
    Fahrzeug()
    {
        vmax = 13.89f;
        v = vmax;
        a = 4.0;
        x = 0;
        y = 0;
        symbol = new FahrzeugSymbol();
        symbol.FarbeSetzen(ZufallsfarbeErzeugen());
        symbol.PositionSetzen(x, y);
    }

    /**
     * Erzeugt eine zufällige Farbe für das Fahrzeug
     */
    private String ZufallsfarbeErzeugen()
    {
        switch ((new java.util.Random()).nextInt (7))
        {
            case 0:
                return "rot";
            case 1:
                return "grün";
            case 2:
                return "blau";
            case 3:
                return "gelb";
            case 4:
                return "cyan";
            case 5:
                return "magenta";
            case 6:
                return "weiss";
            default:
                return "";
        }
    }

    /**
     * Setzt die Position des Fahrzeugs.
     * 
     * @param x x-Position
     * @param y y-Position
     */
    void PositionSetzen(int x, int y)
    {
        this.x = x;
        this.y = y;
        symbol.PositionSetzen (x, y);
    }
    
    /**
     * Fährt das Fahrzeug eine Sekunde nach oben.
     * @param wegmax maximaler Weg bis zum nächsten Hindernis (Ampel oder VOrgänger)
     * @param vvorg Geschwindigkeit des Vorgängerfahrzeugs bzw.der Ampel
     */
    void Fahren(int wegmax, double vvorg)
    {
        double weg;
        double vneu;
        weg = 0;
        for (int i = 0; i < 10; i++)            // Maximaler Weg
        {
            vneu = a * 0.1 + v;
            if (vneu > vmax)
            {
                vneu = vmax;
            }
            weg = weg + (vneu + v) / 2.0 * 0.1;
            v = vneu;
        }
        weg = weg * 5;                         // 1Px = 20cm
        if (wegmax <= 0.0)                     // Muss stehen bleiben
        {
            weg = 0.0;
            v = 0.0;
        }
        else if (weg >= wegmax)                 // Kommt zu dicht an den Vorgänger
        {
            weg = wegmax;
            if (v > vvorg)                      // und ist auch noch zu schnell
            {
                v = vvorg;
            }
        }
        else
        {
            vneu = (wegmax / 5.0 + vvorg * 1.0 - weg / 5.0) / 1.8; // Maximale Geschwindigkeit für den Restabstand zum Vorgänger
            if (vneu < v)                       // Zu schnell für den Abstand
            {
                v = vneu;
            }
        }
        PositionSetzen (x, y - (int) weg);
    }        

    /**
     * Meldet die x-Koordinate zurück
     * @return x-Koordinate
     */
    int XKoordinateGeben()
    {
        return x;
    }

    /**
     * Meldet die y-Koordinate zurück
     * @return y-Koordinate
     */
    int YKoordinateGeben()
    {
        return y;
    }

    /**
     * Meldet die Länge des Fahrzeugs zurück
     * @return Länge
     */
    int LängeGeben()
    {
        return 22;
    }

    /**
     * Meldet die Geschwindigkeit zurück
     * @return Geschwindigkeit
     */
    double GeschwindigkeitGeben()
    {
        return v;
    }

    /**
     * Entfernt das Fahrzeug aus der Anzeige
     */
    void Entfernen()
    {
        symbol.Entfernen();
    }
}
